#define __KERNEL__
#define MODULE
#define EXPORT_SYMTAB
#include <linux/module.h>
#include <linux/init.h>

#include <linux/config.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/sched.h>
#include <linux/smp_lock.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/slab.h>
#include <linux/fcntl.h>
#include <linux/delay.h>
#include <linux/poll.h>
#include <linux/console.h>

#include <asm/io.h>


static ssize_t h_write(struct file * file, const char * buf,
                        size_t count, loff_t *ppos)
{
 unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
 printk("LKM-hello:write minor=%d\n", minor);
 if(count == 0) return 0;
 if(*buf & 1)
    outb(0xF0, 0x3f2); /* Motor ON */
 else
    outb(0x00, 0x3f2); /* Motor OFF */
 return count;
}


static int h_ioctl(struct inode *inode, struct file *file,
                    unsigned int cmd, unsigned long arg)
{
 unsigned int minor = MINOR(inode->i_rdev);
 int retval = 0;

 printk("LKM-hello%d ioctl, cmd: 0x%x, arg: 0x%lx\n", minor, cmd, arg);
 return retval;
}

static int h_open(struct inode * inode, struct file * file)
{
 unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
 printk("LKM-hello:open minor=%d\n", minor);
 return 0;
}

static int h_release(struct inode * inode, struct file * file)
{
 unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
 printk("LKM-hello:release minor=%d\n", minor);
 return 0;
}

static ssize_t h_read(struct file * file, char * buf,
                       size_t count, loff_t *ppos)
{
 unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
 printk("LKM-hello:read minor=%d\n", minor);
 return 0;
}

static struct file_operations hello_fops = {
        owner:          THIS_MODULE,
        write:          h_write,
        ioctl:          h_ioctl,
        open:           h_open,
        release:        h_release,
        read:           h_read,
};

/* -- */

#define HELLO_DEV 121
static unsigned int hello_major;

int init_module()
{int result;
  printk("LKM-hello: Hello\n" );
 
 result = register_chrdev(HELLO_DEV, "hello", &hello_fops);
 if (result < 0) {
   printk("hello: can't get major %d\n",hello_major);
   return result;
 }
 hello_major = HELLO_DEV;
 printk("LKM-hello: major=%d\n", hello_major );
 return 0;
}

void cleanup_module()
{
 printk("LKM-hello:Bye bye, world \n");
 unregister_chrdev(hello_major, "hello");
}

